program AvlSample;

{$APPTYPE CONSOLE}

uses
	Windows,
  SysUtils,
  AVLtrees in '..\AVLtrees.pas';
	
type	
	TMyTree = class(TFloatKeyAVLtree)
	private
		function WriteKey( ANode : TAVLtreeNode; AParm : Pointer ) : Boolean;
	public
		constructor	Create; reintroduce;
	end;

	TMyNode = class(TFloatKeyAVLtreeNode)
	end;

	
var
  sFileName : string;
  oTree : TMyTree;
  oNode : TMyNode;
  F: TextFile;
	fValue : Extended;

//==TMyTree===========================================================
constructor TMyTree.Create;
begin
	inherited Create(TMyNode);
end;
//-------------------------------------------------------------
function TMyTree.WriteKey( ANode : TAVLtreeNode; AParm : Pointer ) : Boolean;
begin
	WriteLn( FloatToStr(TMyNode(ANode).Key) );
	Result := False;	
end;

//-------------------------------------------------------------
var hConsole : THandle = 0;

function ReadChar : Char;
var
	n, nSaveConsoleMode : DWORD;
	cResult : Char;
begin
	if hConsole=0 then hConsole := Windows.GetStdHandle(STD_INPUT_HANDLE);
	GetConsoleMode(hConsole,nSaveConsoleMode);
	try
		SetConsoleMode( hConsole, ENABLE_PROCESSED_INPUT or ENABLE_WINDOW_INPUT or ENABLE_MOUSE_INPUT );
		n := 1;
		if Windows.ReadConsole(hConsole,@cResult,n,n,nil) then Result := cResult else Result := #0;
	finally
		SetConsoleMode(hConsole,nSaveConsoleMode);
	end;
end;

//=============================================================
begin
	// try..except block to handle all exceptions in this console application.
	try	
		sFileName := ParamStr(1);
		if sFileName='' then sFileName := 'input';
		oTree := TMyTree.Create;
		try
			WriteLn('Input:');
			AssignFile(F,sFileName);
			try
				Reset(F);
				while not Eof(F) do
				begin
					Readln(F,fValue);
					WriteLn(FloatToStr(fValue));												
					oNode := TMyNode(oTree.Add(fValue));
					if oNode=nil then
						WriteLn('Value '+FloatToStr(fValue)+' already exists');
				end;
			finally
				CloseFile(F);
			end;
			
			repeat
				WriteLn;
				WriteLn('Keys:');
				oTree.RecursiveBeat(oTree.WriteKey,nil);
				WriteLn;
				WriteLn('Enter command:');
				WriteLn(' a - add;');
				WriteLn(' f - find;');
				WriteLn(' n - find max key less than;');
				WriteLn(' d - delete;');
				WriteLn(' q - exit;');
				case ReadChar of
				'a':
					begin
						Write('Enter new key:');
						ReadLn(fValue);
						oNode := TMyNode(oTree.Add(fValue));
						if oNode=nil then WriteLn('Key is already exists.')
						else WriteLn('New key is inserted.');
					end;
				'f':
					begin
						Write('Enter key:');
						ReadLn(fValue);
						oNode := TMyNode(oTree.Find(fValue));
						if oNode=nil then WriteLn('Key is not found.')
						else WriteLn('Key is found.');
					end;
				'n':
					begin
						Write('Enter value:');
						ReadLn(fValue);
						oNode := TMyNode(oTree.FindMaxKeyLessThan(fValue));
						if oNode=nil then WriteLn('Key is not found.')
						else WriteLn('Nearest key: '+FloatToStr(oNode.Key));
					end;
				'd':
					begin
						Write('Enter key to delete:');
						ReadLn(fValue);
						oNode := TMyNode(oTree.Find(fValue));
						if oNode=nil then WriteLn('Key is not found.')
						else
							begin
								oNode.Destroy;
								WriteLn('Key is deleted.');
							end;
					end;
				'q', 'x': Exit;
				else
					WriteLn('Invalid command.');
				end;					
			until False;
			
			
		finally
			FreeAndNil(oTree);
		end;
		
	except
		on E:Exception do WriteLn(E.ClassName+': '+E.Message);
	end;
end.


